﻿using System;
using System.Collections.Generic;
using UnityEngine;

public class ConversationManager : MonoBehaviour {

    private static ConversationManager instance;

    public static ConversationManager Instance()
    {
        if (instance == null)
            instance = GameObject.Find("ConversationManager").GetComponent<ConversationManager>();
        if (instance == null)
            Debug.LogError("Can't find ConversationManager!");

        return instance;
    }

	// A map of names to conversations
	public Dictionary<string, int> nameToConversationMap;
	// Conversations as a list so that we can see them in Unity's inspector
	public List<Conversation> conversations;
	// The first conversation to load
	public string ConvoToLoad;
	// Use this for initialization
	void Start () {
		// Initialse our arrays
		nameToConversationMap = new Dictionary<string, int>();
		conversations = new List<Conversation>();
		
        DisplayManager.Instance().sendResponseIntro(ConvoToLoad);
        DisplayManager.currentConversation = "february";
	}
    public void loadIntroText(){
        DisplayManager.Instance().sendResponseIntro(ConvoToLoad);
    }
	public void loadDefaultConversation(){
		LoadConversation("february", ConvoToLoad);
	}

	// Accept a name, and conversation to load.
	public void LoadConversation(string personName, string conversationName){
		DisplayManager.Instance().introContainer.SetActive(false);
		DisplayManager.Instance().messengerWindow.SetActive(true);
		DisplayManager.currentConversation = personName;
		Debug.Log("Loading conversation: " + conversationName);
		// Load the conversation
		Type conversationClass = Type.GetType(conversationName);
        Conversation convo = Activator.CreateInstance(conversationClass) as Conversation;

		conversations[nameToInt(personName)] = convo;
		progressConversation(personName); //Kickstart the conversation :3
	}
	int nameToInt(string name){
		int returnValue = 0;
		// Initialise the NPC's name if it's the first time we're referencing them
		if(!nameToConversationMap.ContainsKey(name)){
			returnValue = conversations.Count;
			nameToConversationMap.Add(name,  returnValue);
			// Set Save the conversation to the NPC's name
			conversations.Add(new Conversation());
		}else{
			// Set Save the conversation to the NPC's name
			returnValue = nameToConversationMap[name];
		}
		return returnValue;
	}
	public void progressConversation(string personName){
		Conversation convo = conversations[nameToInt(personName)];
		if (convo.interactions.Count>0)
		{
			for (int i = 0; i < convo.interactions[0].messages.Count; i++)
			{
				DisplayManager.Instance().addNarrativeBlock(personName, convo.interactions[0].messages[i]);
			}
			DisplayManager.Instance().addNarrativeBlock(personName, convo.interactions[0].answer);
			convo.interactions.RemoveAt(0);
		}else{
			convo.finish();
			if(convo.nextStep != null && convo.nextStep!="")
				LoadConversation(personName, convo.nextStep);
		}
	}

	// Update is called once per frame
	void Update () {
		
	}
}
