﻿using System.Collections.Generic;
using UnityEngine;
using System;

public class DisplayManager : MonoBehaviour {
     
     private Dictionary<string, ChatDisplay> chatDisplays;
     public GameObject textPrefab;
     public GameObject answerPrefab;
     public GameObject playerMessagePrefab;
     public GameObject chatContainerPrefab;
     public GameObject messengerWindow;

     public GameObject introChoicePrefab;
     public GameObject introContainer;
     
    private static DisplayManager instance;

    public GameObject parentCanvas;
    public GameObject chatPanePrefab;
    private GameObject chatPane;

    public static DisplayManager Instance()
    {
        if (instance == null)
            instance = GameObject.Find("DisplayManager").GetComponent<DisplayManager>();
        if (instance == null)
            Debug.LogError("Can't find StateManage!");

        return instance;
    }

    private List<object> displayList;
    public void loadIntroMessage(IntroMessage messageText){
        messengerWindow.SetActive(false);
        displayIntroMessage(messageText);
        
    }
    public void displayIntroMessage(IntroMessage message){
          for (int i = introContainer.transform.childCount-1; i >= 0; i--)
          {
              Destroy(introContainer.transform.GetChild(i).gameObject);
          }
          GameObject introObject = Instantiate(introChoicePrefab);
            introContainer.SetActive(true);
          introObject.transform.SetParent(introContainer.transform);
          introObject.GetComponent<IntroUI>().setInterUI(message);
    }
    void Start(){
        chatPane = Instantiate(chatPanePrefab);
        chatPane.transform.SetParent(parentCanvas.transform);
        chatPane.transform.SetSiblingIndex(0);
        RectTransform rect = chatPane.GetComponent<RectTransform>();
		rect.sizeDelta = new Vector2(0f, 0f);
		rect.offsetMin = new Vector2(0f, 180f);
		rect.offsetMax = new Vector2(0f, -140f);
        messengerWindow = chatPane.transform.FindChild("Viewport").FindChild("Content").gameObject;
    }
    public static string currentConversation;
	public void sendResponseIntro(string response){
        if(response == null || response == "")
            return;

        Debug.Log("sending response " + response.ToString() + " " +Type.GetType(response));
        Debug.Log("checking " + Type.GetType(response) + " == " + typeof(Conversation) + " = " + (Type.GetType(response) == typeof(Conversation)));
        try{
            Type conversationClass = Type.GetType(response);
            IntroText messageText = Activator.CreateInstance(conversationClass) as IntroText;
            loadIntroMessage(messageText.setup());
        }catch{
            ConversationManager.Instance().LoadConversation(currentConversation, response);
        }
        //displayIntroMessage(response);
	}
	public void sendResponse(string personName, Message response, PlayerMessage playerLongAnswer){
        Debug.Log("sending response " + response.ToString());

        checkChatDisplayExists(personName);
        chatDisplays[personName].sendResponse(response, playerLongAnswer);
	}
    public void addNarrativeBlock(string personName, object block)
    {
        checkChatDisplayExists(personName);
        chatDisplays[personName].addNarrativeBlock(block);
    }
    public void hideChat(string personName)
    {

        checkChatDisplayExists(personName);
        chatDisplays[personName].chatContainer.SetActive(false);
        chatDisplays[personName].nextDisplay = chatDisplays[personName].nextDisplay + (60 * 60 * 24);
    }
    public void showChat(string personName)
    {
		DisplayManager.currentConversation = personName;
        checkChatDisplayExists(personName);
        chatDisplays[personName].chatContainer.gameObject.SetActive(true);
        messengerWindow.SetActive(true);
    }
    public void clearChat(string personName)
    {
        checkChatDisplayExists(personName);
        chatDisplays[personName].clearChat();
    }

    public void checkChatDisplayExists(string name){
        if(chatDisplays == null){
            chatDisplays = new Dictionary<string, ChatDisplay>();
        }
        if(!chatDisplays.ContainsKey(name)){
            GameObject chatContainer = Instantiate(chatContainerPrefab);
            chatContainer.transform.SetParent(messengerWindow.transform);
            ChatDisplay display = gameObject.AddComponent<ChatDisplay>() as ChatDisplay;

            RectTransform displayTransform = chatContainer.GetComponent<RectTransform>();
            displayTransform.offsetMin = new Vector2(0f, 0f);
            displayTransform.offsetMax = new Vector2(0f, 0f);
            displayTransform.sizeDelta = new Vector2(0f, 0f);

            chatDisplays.Add(name, display);
            chatDisplays[name].chatName = name;
            chatDisplays[name].chatContainer = chatContainer;
            chatDisplays[name].textPrefab = textPrefab;
            chatDisplays[name].answerPrefab = answerPrefab;
            chatDisplays[name].playerMessagePrefab = playerMessagePrefab;
        }
    }

}